@@ -234,7 +234,7 @@ src_presentation_read_formated_string_from_default (const gchar *device,
     else if (strcmp (device, "speech") == 0)
 	return g_strdup ("%name<voice=name>%%role<voice=role>%");
     else
-	return g_strdup ("%object<AlignmentX=\"0\" AlignmentY=\"0\">%");
+	return g_strdup ("%object%");
 
     return NULL;
 }
@@ -1363,7 +1363,8 @@ static struct
         {"extended",			"EXT",	"EXTENDED",		"Extended",		},
 	{"table-line",			"TLI",	"TABLE LINE",		"Table line",		},
         {"table-columns-header",	"TCH","TABLE COLUMNS HEADER",	"Table columns header",	},
-	{"title-bar",			"TIT",	"TITLE BAR",		"Title bar",		}
+	{"title-bar",			"TIT",	"TITLE BAR",		"Title bar",		},
+	{"edit-bar",			"EDB",	"EDIT BAR",		"Edit bar",		}
     };
 
 static gint
@@ -1657,8 +1658,8 @@ src_presentation_get_info_from_sro (SRObject *obj,
 
 static gchar*
 src_presentation_present_sro_for_device_ (SRObject *obj,
-					 const gchar *device,
-					 SRCPresentationItem **chunk)
+					  const gchar *device,
+					  SRCPresentationItem **chunk)
 {
     gint i;
     gchar *rv;
@@ -1693,33 +1694,42 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 		    
 		    add = g_strdup ("");		    
 		    sro_get_children_count (obj, &cnt);
-		    for (j = 0; j < cnt; j++)
+		    if (!sro_manages_descendants (obj)) 
 		    {
-			SRObject *child;
-			gchar *tmp, *tmp2, *tmp3;
-
-			if (!sro_get_i_child (obj, j, &child))
-			    continue;
-			
-			tmp = add;
-			tmp2 = src_presentation_present_sro_for_device_ (child, device, (SRCPresentationItem **)chunk[i]->data1);
-			
-			tmp3 = NULL;
-			if (j != cnt - 1 && chunk[i]->data2)
+		        for (j = 0; j < cnt; j++)
 			{
-			    gchar *tmp4;
-			    tmp4 = g_strdup_printf ("dot%s", (gchar *)chunk[i]->data2);
-			    if (tmp4)
+			    SRObject *child;
+			    gchar *tmp, *tmp2, *tmp3;
+			  
+			    if (!sro_get_i_child (obj, j, &child))
+			      continue;
+			    
+			    tmp = add;
+			    tmp2 = src_presentation_present_sro_for_device_ (child, device, (SRCPresentationItem **)chunk[i]->data1);
+			    
+			    tmp3 = NULL;
+			    if (j != cnt - 1 && chunk[i]->data2)
 			    {
-				tmp3 = src_xml_make_part ("DOTS", NULL, tmp4);
-				g_free (tmp4);
+			        gchar *tmp4;
+				tmp4 = g_strdup_printf ("dot%s", (gchar *)chunk[i]->data2);
+				if (tmp4)
+				{
+				    tmp3 = src_xml_make_part ("DOTS", NULL, tmp4);
+				    g_free (tmp4);
+				}
 			    }
+			    add = g_strconcat (add, tmp2, tmp3, NULL);
+			    g_free (tmp);
+			    g_free (tmp2);
+			    g_free (tmp3);
+			    sro_release_reference (child);
 			}
-			add = g_strconcat (add, tmp2, tmp3, NULL);
-			g_free (tmp);
-			g_free (tmp2);
-			g_free (tmp3);
-			sro_release_reference (child);
+		    }
+		    else
+		    {
+		      /* FIXME: presentation of first N visible children?
+			 Or just a notation of "Object contains %d children" ?
+		      */
 		    }
 		}
 		break;
@@ -1908,8 +1918,10 @@ src_presentation_present_sro_for_magnifier (SRObject *obj,
 		    gchar *tmp1, *tmp2, *alg;
 		    tmp1 = strstr (string, "<");
 		    tmp2 = strstr (string, ">");
-		    sru_assert (tmp1 && tmp2);
-		    alg = g_strndup (tmp1 + 1, tmp2 - tmp1 - 1);
+		    if (tmp1 && tmp2)
+			alg = g_strndup (tmp1 + 1, tmp2 - tmp1 - 1);
+		    else
+			alg = g_strdup ("");
 		    presentation_ = g_strdup_printf ("ROILeft =\"%d\" ROITop =\"%d\" ROIWidth =\"%d\" ROIHeight=\"%d\" %s",
 					location.x,
 					location.y,
@@ -1939,7 +1951,37 @@ src_presentation_present_sro_for_speech (SRObject *obj,
 
     if (sro_get_role_name (obj, &role, SR_INDEX_CONTAINER))
     {
-	if (src_presentation_find_chunk_for_speech (role, reason, &presentation))
+	if (strcmp (reason, "window:create") == 0 && strcmp (role, "alert") == 0)
+	{
+	    gchar *title, *text, *button, *add1, *add2, *add3;
+
+	    title = text = button = NULL;
+	    add1 = add2 = add3 = NULL;
+	    sro_alert_get_info (obj, &title, &text, &button);
+	    if (title)
+	    {
+		add1 = src_xml_format ("TEXT", src_speech_get_voice ("name"), title);
+		SR_freeString (title);
+	    }
+	    if (text)
+	    {
+		add2 = src_xml_format ("TEXT", src_speech_get_voice ("name"), text);
+		SR_freeString (text);
+	    }
+	    if (button)
+	    {
+		add3 = src_xml_format ("TEXT", src_speech_get_voice ("name"), button);
+		SR_freeString (button);
+	    }
+	    if (add1 || add2 || add3)
+	    {
+		presentation_ = g_strconcat (add1 ? add1 : "", add2 ? add2 : "", add3 ? add3 : "", NULL);
+	    }
+	    g_free (add1);
+	    g_free (add2);
+	    g_free (add3);
+	}
+	else if (src_presentation_find_chunk_for_speech (role, reason, &presentation))
 	{
 	    presentation_ = src_presentation_present_sro_for_device_ (obj, device, presentation);
 	}
